//+------------------------------------------------------------------+
//|                                                 ZeroLag_MACD.mq5 |
//|                                                        avoitenko |
//|                        https://login.mql5.com/en/users/avoitenko |
//+------------------------------------------------------------------+
#property copyright "avoitenko"
#property link      "https://login.mql5.com/en/users/avoitenko"
#property version   "3.00"

//---
#include <MovingAverages.mqh>
//---
#property indicator_separate_window
#property indicator_buffers 8
#property indicator_plots   2

//--- plot MACDBuffer
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  clrBlue
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2

//--- plot SignalBuffer
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1

//---- input parameters
input ushort InpFastEMA    = 12;// Fast EMA
input ushort InpSlowEMA    = 26;// Slow EMA
input ushort InpSignalEMA  =  9;// Signal EMA
input ENUM_APPLIED_PRICE InpApplyPrice=PRICE_CLOSE;// Apply to

//---- buffers
double MACDBuffer[];
double SignalBuffer[];
double FastEMABuffer[];
double SlowEMABuffer[];
double val[];
double val1[];
double val2[];
double val3[];

//---indicator handles
int maHandle1,maHandle2;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   SetIndexBuffer(0,MACDBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,SignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,FastEMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(3,SlowEMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,val,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,val1,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,val2,INDICATOR_CALCULATIONS);
   SetIndexBuffer(7,val3,INDICATOR_CALCULATIONS);

   string name=StringFormat("ZeroLag MACD(%d,%d,%d)",InpFastEMA,InpSlowEMA,InpSignalEMA);
   IndicatorSetString(INDICATOR_SHORTNAME,name);

   PlotIndexSetString(0,PLOT_LABEL,name);
   PlotIndexSetString(1,PLOT_LABEL,"Signal");

   ArraySetAsSeries(MACDBuffer,true);
   ArraySetAsSeries(SignalBuffer,true);
   ArraySetAsSeries(FastEMABuffer,true);
   ArraySetAsSeries(SlowEMABuffer,true);
   ArraySetAsSeries(val,true);
   ArraySetAsSeries(val1,true);
   ArraySetAsSeries(val2,true);
   ArraySetAsSeries(val3,true);

//--- handles
   maHandle1=iMA(NULL,0,InpFastEMA,0,MODE_EMA,InpApplyPrice);
   if(maHandle1==INVALID_HANDLE) {Print("Error initialize iMA handle 1");return(-1);}

   maHandle2=iMA(NULL,0,InpSlowEMA,0,MODE_EMA,InpApplyPrice);
   if(maHandle2==INVALID_HANDLE) {Print("Error initialize iMA handle 2");return(-1);}
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {

   int limit;
   if(prev_calculated>rates_total || prev_calculated<=0)limit=rates_total-1;
   else limit=rates_total-prev_calculated;

   if(CopyBuffer(maHandle1, MAIN_LINE, 0, rates_total, FastEMABuffer) != rates_total) return(0);
   if(CopyBuffer(maHandle2, MAIN_LINE, 0, rates_total, SlowEMABuffer) != rates_total) return(0);

   ExponentialMAOnBuffer(rates_total,prev_calculated,0,InpFastEMA,FastEMABuffer,val);
   ExponentialMAOnBuffer(rates_total,prev_calculated,0,InpSlowEMA,SlowEMABuffer,val1);

   for(int i=limit; i>=0 && !IsStopped(); i--)
     {
      double ZeroLagEMAp = FastEMABuffer[i] + FastEMABuffer[i] - val[i];
      double ZeroLagEMAq = SlowEMABuffer[i] + SlowEMABuffer[i] - val1[i];
      MACDBuffer[i]=ZeroLagEMAp-ZeroLagEMAq;
     }

   ExponentialMAOnBuffer(rates_total,prev_calculated,0,InpSignalEMA,MACDBuffer,val2);
   ExponentialMAOnBuffer(rates_total,prev_calculated,0,InpSignalEMA,val2,val3);

   for(int i=limit; i>=0 && !IsStopped(); i--)
      SignalBuffer[i]=val2[i]+val2[i]-val3[i];

   return(rates_total);
  }
//+------------------------------------------------------------------+
